#!/usr/bin/env python

import os
import sys
import re
import shutil

from Bio import SeqIO
from collections import defaultdict


#####
# this is the script for species recognition using 40 mgs
#####
# what this script does
##
#
# 0. rename proteins and genes
#
# 1. extract marker genes from a genome --> using fetchMGs [using best hit +  cutoffs]
#
# 2. pre-screen distances using usearch 
#    usearch -ublast query.fasta -db db.fasta -id 0.85 --strand plus -blast6out COG0085.noX.project.usearch_search_global.m8
#
# 3. extract all clusters that have at least one hit above 85% ID
#
# 4. calculate distances from input marker genes to all markere genes from the extracted clusters --> using glsearch
#
# 5. calculate average distance to clusters
#
# 6. calculate mean average distance to clusters; get closest cluster; determine which cutoffs are fulfilled; 
#
# 7. generate results html file
#
# 8. report short results
#
# db: all genes in format taxID.projectID in files COGXXX.fna and in COGXXX/taxID.projectID.fna
# genelenghts: 1 file with: COGXXX \t taxID.projectID

#ATTENTION: Please set the bin folder according to your installation
binFolder = "."


def reformatGenome(infileGenes, infileProteins, outfileGenes, outfileProteins):

	dictNewNames = {}
	dictNewNames2OldNames = {}	
	intCounter = 1
	new_seq_genes = []
	for record in SeqIO.parse(open(infileGenes,'rU'),'fasta'):
		new_record_name = "INPUT." + str(intCounter)
		dictNewNames[record.id] = new_record_name
		dictNewNames2OldNames[new_record_name] = record.id
		record.id = new_record_name
		record.name = ''
		record.description = ''
		new_seq_genes.append(record)
		intCounter += 1
		
	new_seq_proteins = []
	for record in SeqIO.parse(open(infileProteins,'rU'),'fasta'):
		if (record.id in dictNewNames):
			new_record_name = dictNewNames[record.id]
			record.id = new_record_name
			record.name = ''
			record.description = ''
			new_seq_proteins.append(record)
		else:
			sys.exit(record.id + " is missing protein sequence. Please check your files")
			
	SeqIO.write(new_seq_genes, outfileGenes, 'fasta')	
	SeqIO.write(new_seq_proteins, outfileProteins, 'fasta')
	
	return(dictNewNames2OldNames)

def runFetchMGs(fileGenes, fileProteins, outfolder, threads):

	cmd = "perl " + binFolder + "/" + "MOCATFetchMGs03.pl -v -t " + str(threads) + " -c all -o " + outfolder + " -m extraction -d " + fileGenes + " " + fileProteins
	os.system(cmd)	


#for 2.
def runUsearchPreScreen(listOfCOGs, fastaFolder, usearchFolder, strDB_OGfolder, threads):

	listHitTaxIDProjects = []

	for strCOGs in listOfCOGs:
	
		strQueryFasta = fastaFolder + "/" + strCOGs + ".fna"
		strOutfile = usearchFolder + "/" + strCOGs + ".m8"
		strDBfasta = strDB_OGfolder + "/" + strCOGs + ".fna"
		
		cmd = "usearch -threads " + threads + " -quiet -search_global " + strQueryFasta + " -db " + strDBfasta + " -id 0.85 --strand plus -blast6out " + strOutfile
		print(cmd)
		os.system(cmd)
		
		fileOutfile = open(strOutfile, "r")

		for strCurrentLine in fileOutfile:
			strCurrentLine = strCurrentLine.strip()
			arrCurrentLine = strCurrentLine.split('\t')
			
			strQueryName = arrCurrentLine[0]
			strSubjectName = arrCurrentLine[1]
			
			listHitTaxIDProjects.append(strSubjectName)
			
	listHitTaxIDProjects = list(set(listHitTaxIDProjects))
			
	return(listHitTaxIDProjects)

	
def fillClusterDict(clusterFileName):
	clusterFile = open(clusterFileName, "r")
	
	dictCluster2IDs = defaultdict(list)	
	dictID2Cluster = {}

	for strClusterLine in clusterFile:
		strClusterLine = strClusterLine.rstrip('\n')
		arrClusterLine = strClusterLine.split('\t')
		
		strClusterID = arrClusterLine[0]
		
		for strID in arrClusterLine[1:]:
			if (not strID in dictID2Cluster):
				dictID2Cluster[strID] = strClusterID
				dictCluster2IDs[strClusterID].append(strID)
				
			else:
				print(strID + " " + strClusterID + " assigned to multiple clusters in " + clusterFileName + ".")			

	return(dictID2Cluster, dictCluster2IDs)	
	
	
def getAllClusterMembers(listHitTaxIDProjects, dictTaxIDProject2Cluster, dictCluster2IDs):

	listHitClusters = []
	listHitTaxIDProjects_FullClusters = []
	
	for strTaxIDProjects in listHitTaxIDProjects:
		
		strCurrentCluster = dictTaxIDProject2Cluster[strTaxIDProjects]
		listHitClusters.append(strCurrentCluster)
	
	listHitClusters = list(set(listHitClusters))
		
	for strCurrentCluster in listHitClusters:
		
		listdictTaxIDProjectsOfCluster = dictCluster2IDs[strCurrentCluster]
		listHitTaxIDProjects_FullClusters.extend(listdictTaxIDProjectsOfCluster)
		
	listHitTaxIDProjects_FullClusters = list(set(listHitTaxIDProjects_FullClusters))	
		
	return(listHitTaxIDProjects_FullClusters, listHitClusters)

	
def getGeneLengthInput(fastaFolder, listOfCOGs, dictCOG2Gene2Length):
		
	for strCOG in listOfCOGs:	
		strQueryFasta = fastaFolder + "/" +  strCOG + ".fna"	

		for record in SeqIO.parse(open(strQueryFasta, "rU"), "fasta"):
			intSeqLength = len(record.seq)
			strGeneName = record.id
			
			dictCOG2Gene2Length[strCOG]["INPUT"] = intSeqLength
		
	return(dictCOG2Gene2Length)
			
	
def parseGene2Lengths(geneLengthFileName, dictCOG2Gene2Length):
	geneLengthFile = open(geneLengthFileName, "r")

	for strGeneLengthLine in geneLengthFile:
	
		strGeneLengthLine = strGeneLengthLine.rstrip('\n')
		arrGeneLengthLine = strGeneLengthLine.split('\t')	

		strCOG = arrGeneLengthLine[0]
		strGeneName = arrGeneLengthLine[1]
		strLength = arrGeneLengthLine[2]
		
		intLength = int(strLength)
	
		dictCOG2Gene2Length[strCOG][strGeneName] = intLength
		
	return(dictCOG2Gene2Length)	

def parseCutoffs(cutoffFileName):
	cutoffFile = open(cutoffFileName, "r")
	dictCutoffs = {}
	for strCutoffLine in cutoffFile:
	
		strCutoffLine = strCutoffLine.rstrip('\n')
		arrCutoffLine = strCutoffLine.split('\t')	

		strID = arrCutoffLine[0]
		strCutoff= arrCutoffLine[1]
		
		floatCutoff = float(strCutoff)
	
		dictCutoffs[strID] = floatCutoff
		
	return(dictCutoffs)		

def parseTaxonomyMapping(taxonomyFile):
	cutoffFile = open(taxonomyFile, "r")
	dictTaxonomy = {}
	for strCutoffLine in cutoffFile:
	
		strCutoffLine = strCutoffLine.rstrip('\n')
		arrCutoffLine = strCutoffLine.split('\t')
		
		strID = arrCutoffLine[0]
		strTaxonomy= arrCutoffLine[1]
		
		dictTaxonomy[strID] = strTaxonomy	
	return(dictTaxonomy)		
	
def convertPercID2Distance(floatInput):
	floatOutput = (100.0 - float(floatInput))/100.0
	return(floatOutput)
	
	
def convertDistance2PercID(floatInput):
	floatOutput = (1 - float(floatInput))*100.0
	return(floatOutput)
		
	
def runGLsearch(listHitTaxIDProjects_FullClusters, fastaFolder, listOfCOGs, DBfolder, distFolder, dictCOG2Gene2Length, threads):

	dictInput_ID2COG2Distance = defaultdict(lambda: defaultdict(float))
	dictInput_ID2COG2DistanceWeight = defaultdict(lambda: defaultdict(float))
	
	for strCOG in listOfCOGs:
	
		strQueryFasta = fastaFolder + "/" +  strCOG + ".fna"
		GLsearchFileName = distFolder + "/" + strCOG + ".glsearch"
		fileOutfile = open(GLsearchFileName, "w")	
		fileOutfile.close()
	
		for strHitID in listHitTaxIDProjects_FullClusters:
			
			strSubjectFasta = DBfolder + "/" + strCOG + "/" + strCOG + "." + strHitID + ".fna"
			if (os.path.isfile(strSubjectFasta)):
				intQueryLength = dictCOG2Gene2Length[strCOG]["INPUT"]	
				intSubjectLength = dictCOG2Gene2Length[strCOG][strHitID]
			###checking which seq is longer and only align short vs long
			#this is for sequential alignments -- parallellization needs some changes (4 processors are also used here)
				if (intQueryLength <= intSubjectLength):
					cmd = "glsearch36 -T " + threads+ " -n -3 -m 8 -E 1000 -b 10000 -d 0 " + strQueryFasta + " " + strSubjectFasta + " >> " + GLsearchFileName
					os.system(cmd)
				else:
					cmd = "glsearch36 -T " + threads + " -n -3 -m 8 -E 1000 -b 10000 -d 0 " + strSubjectFasta + " " + strQueryFasta + " >> " + GLsearchFileName
					os.system(cmd)
		
		(dictInput_Gene2Distance, dictInput_Gene2DistanceWeight) = parseGLsearchResults(GLsearchFileName, dictCOG2Gene2Length, strCOG)
		
		for strGene in dictInput_Gene2Distance:
			dictInput_ID2COG2Distance[strGene][strCOG] = dictInput_Gene2Distance[strGene]
			dictInput_ID2COG2DistanceWeight[strGene][strCOG] = dictInput_Gene2DistanceWeight[strGene]
		
	return(dictInput_ID2COG2Distance, dictInput_ID2COG2DistanceWeight)

		
def parseGLsearchResults(GLsearchFileName, dictCOG2Gene2Length, strCOG):

	#dictInput_Gene2PercID = defaultdict(float)
	dictInput_Gene2Distance = {}
	dictInput_Gene2Matches = {}
	dictInput_Gene2DistanceWeight = {}
	
	
	GLsearchFile = open(GLsearchFileName, "r")
#	print(GLsearchFileName)
	for strGLsearchLine in GLsearchFile:
		if(strGLsearchLine[0] != '#'):
			strGLsearchLine = strGLsearchLine.rstrip('\n')
			arrGLsearchLine = strGLsearchLine.split('\t')
			
			strQueryID = arrGLsearchLine[0]
			strSubjectID = arrGLsearchLine[1]
			strPercID = arrGLsearchLine[2]
			strAlignmentLength = arrGLsearchLine[3]
			strMismatches = arrGLsearchLine[4]
			strGapOpens = arrGLsearchLine[5]
			
			intAlignmentLength = int(strAlignmentLength)
			intMismatches = int(strMismatches)
			intGapOpens = int(strGapOpens)
			intMatches = intAlignmentLength - intMismatches - intGapOpens
			floatPercID = float(strPercID)
			
			strQueryCheck = strQueryID.split('.')[0]
			strSubjectCheck = strSubjectID.split('.')[0]
			
			# the input should always be the query; the input is renamed to "INPUT", so the length can be extracted
			if (strQueryCheck == "INPUT"):
				strNewSubjectID = strSubjectID
				strQueryID = "INPUT"
			elif (strSubjectCheck == "INPUT"):
				strNewSubjectID = strQueryID			
				strSubjectID = "INPUT"			
			
			intQueryLength = dictCOG2Gene2Length[strCOG][strQueryID]	
			intSubjectLength = dictCOG2Gene2Length[strCOG][strSubjectID]
			
			if (intQueryLength == 0):
				print(strQueryID + " " + strCOG + " " + strNewSubjectID + strSubjectID)
			
			if (intQueryLength <= intSubjectLength):
				if (not strNewSubjectID in dictInput_Gene2Distance):
	#				dictInput_Gene2PercID[strNewSubjectID] = floatPercID
					dictInput_Gene2Distance[strNewSubjectID] = (100.0 - floatPercID)/100.0
					dictInput_Gene2Matches[strNewSubjectID] = intMatches
					dictInput_Gene2DistanceWeight[strNewSubjectID] = intQueryLength
					
				elif(dictInput_Gene2Matches[strNewSubjectID] < intMatches):
#					dictInput_Gene2PercID[strNewSubjectID] = floatPercID
					dictInput_Gene2Distance[strNewSubjectID] = (100.0 - floatPercID)/100.0
					dictInput_Gene2Matches[strNewSubjectID] = intMatches
					dictInput_Gene2DistanceWeight[strNewSubjectID] = intQueryLength

	return(dictInput_Gene2Distance, dictInput_Gene2DistanceWeight)

			
def calcDist2Clusters(dictInput_ID2COG2Distance, dictInput_ID2COG2DistanceWeight, listHitTaxIDProjects_FullClusters, listHitClusters, dictCluster2IDs ):

	dictAverageDistances = {}

	#strID = taxProjectID
	for strID in listHitTaxIDProjects_FullClusters:
		listWeightedDistances = []
		listWeights = []
		for strCOG in dictInput_ID2COG2Distance[strID]:
			floatCurrDistance = dictInput_ID2COG2Distance[strID][strCOG]
			floatCurrWeight = dictInput_ID2COG2DistanceWeight[strID][strCOG]
			if (not floatCurrWeight):
				print(strID + " " + strCOG + " " + str(dictInput_ID2COG2DistanceWeight[strID][strCOG]))
			
			floatCurrWeightedDistance = floatCurrDistance*floatCurrWeight
			
			listWeightedDistances.append(floatCurrWeightedDistance)		
			listWeights.append(floatCurrWeight)
			
		averageWeightedDistance = 0
		
		if(sum(listWeights) == 0):
			print(strID + " not ok")

		if(sum(listWeights) > 0):
			averageWeightedDistance = sum(listWeightedDistances)/sum(listWeights)
			dictAverageDistances[strID] = averageWeightedDistance

	#result: dictAverageDistances (this dict has distances from input to different genomes)
			

	floatClosestClusterDistance = 100000.0
	strClosestCluster = ""
	
	for strCluster in listHitClusters:
		listIDsOfCluster = dictCluster2IDs[strCluster]
		listDistToCluster = []
		
		for strID in listIDsOfCluster:
			if (strID in dictAverageDistances):
				floatAvgDist = dictAverageDistances[strID]
				listDistToCluster.append(floatAvgDist)
			
		floatAverageClusterDistance = sum(listDistToCluster)/len(listDistToCluster)
		
		if (floatAverageClusterDistance < floatClosestClusterDistance ):
			floatClosestClusterDistance = floatAverageClusterDistance
			strClosestCluster = strCluster

	#result: floatClosestClusterDistance (distance to closest cluster)
	#result: strClosestCluster (id of closest cluster)
			
	dictClosestClusterGenomeDistances = defaultdict(float)
	dictClosestClusterOGDistancesLists = defaultdict(list)
	
	listIDsOfClosestCluster = dictCluster2IDs[strClosestCluster]
	for strID in listIDsOfClosestCluster:
		dictClosestClusterGenomeDistances[strID] = dictAverageDistances[strID]
		#print("BEST CLUSTER " + strID + "\t" + str(dictAverageDistances[strID]))
		for strCOG in dictInput_ID2COG2Distance[strID]:
			floatCurrDistance = dictInput_ID2COG2Distance[strID][strCOG]
			
			dictClosestClusterOGDistancesLists[strCOG].append(floatCurrDistance)
	
	#result: dictClosestClusterGenomeDistances (distances to all genomes in closest cluster)
	#result: dictClosestClusterOGDistancesLists (lists of distances to each OG)

	
	dictClosestClusterOGDistances = defaultdict(float)	
	for strCOG in dictClosestClusterOGDistancesLists:
		listDistToCluster = dictClosestClusterOGDistancesLists[strCOG]
		floatAverageClusterDistance = sum(listDistToCluster)/len(listDistToCluster)
		dictClosestClusterOGDistances[strCOG] = floatAverageClusterDistance
	#result: dictClosestClusterOGDistances (average distances to each OG)
	
	return(strClosestCluster, floatClosestClusterDistance, dictClosestClusterGenomeDistances, dictClosestClusterOGDistances)

	



def writeHeader(outfileName):
		
	outfile = open(outfileName, "w")	
	#currenttime = 
	
	outfile.write('<!DOCTYPE html>\n')
	outfile.write('<html>\n')
	outfile.write('<head>\n')
	outfile.write('\t<meta charset="utf-8" />\n')
#	outfile.write('\t<meta name="author" content="Daniel R. Mende" />\n')
#	outfile.write('\t<meta name="keywords" content="Species Assignments" />\n')
#	outfile.write('\t<meta name="description" content="Bacterial and Archaeal Species Assignments using 40 universal single copy marker genes" />\n')
	outfile.write('\t<title>Bacterial and Archaeal Species Assignments using 40 universal single copy marker genes</title>\n')
	#outfile.write('	<time>' + 2013-03-01 + '</time>\n')
	outfile.write('\t<link rel="stylesheet" href="./main.css" type="text/css" media="screen" charset="utf-8" />\n')

	outfile.write('</head>\n')
	outfile.write('\n')	
	outfile.write('<body>\n')
	outfile.write('\n')	
	outfile.write('<div id="container">\n')
	outfile.write('\t<div id="top-bar">\n')
	outfile.write('\t\t<ul id="top-bar">\n')
	#outfile.write('\t\t\t<li class="top-bar-left"><a href="./index.html">Home</a></li>\n')
	#outfile.write('\t\t\t<li class="top-bar-right"><a href="./citation.html">Citation</a></li>\n')
	#outfile.write('\t\t\t<li class="top-bar-right"><a href="./markergenes.html">Marker Genes</a></li>\n')
	#outfile.write('\t\t\t<li class="top-bar-right"><a href="./overview.html">Overview</a></li>\n')
	outfile.write('\n')
	outfile.write('\t\t</ul>\n')
	outfile.write('\t</div>	\n')
	outfile.close()

def writeOverviewResults(outfileName, strClosestCluster, floatClosestClusterDistance, dictCluster2Taxonomy, dictCluster2IDs, dictCutoffs):

	outfile = open(outfileName, "a")	

	strClusterTaxonomy = dictCluster2Taxonomy[strClosestCluster]
	strClosestClusterDistance = str(floatClosestClusterDistance)
	floatCutoff = dictCutoffs["all"]
	strCutoff = str(floatCutoff)
	floatClosestClusterPercID = convertDistance2PercID(floatClosestClusterDistance)	
	strClosestClusterPercID = str(floatClosestClusterPercID)	
	strClusterSize = str(len(dictCluster2IDs[strClosestCluster]))
	#still needed

	
	strColorLine ='<td style="background-color:Salmon;">Below cutoff</td>'
	strBottomLine = '<font size="+1"> Cluster could not be assigned to a species cluster</font>'
	
	if (floatClosestClusterDistance <= floatCutoff):
		strColorLine = '<td style="background-color:lightgreen;">Above cutoff</td>'
		strBottomLine = '<font size="+1"> Cluster was succesfully assigned to a species cluster	</font>'
		
	#write html code
	
	outfile.write('<div id="overviewResults">\n')
	outfile.write('\t<p>\n')
	outfile.write('\t<font size="+1"> Following cluster is most similar to the input genome:</font>\n')
	outfile.write('\t<table cellpadding="5">\n')
	
	outfile.write('\t<tr>\n')	
	outfile.write('\t<th>Cluster Taxonomy:</th>\n')	
	outfile.write('\t<td>' + strClusterTaxonomy + '</td>\n')
	#something to write in the 3rd column?
	outfile.write('\t<td></td>\n')	
	outfile.write('\t</tr>\n')		

	outfile.write('\t<tr>\n')	
	outfile.write('\t<th>Average % Identity:</th>\n')	
	outfile.write('\t<td>' + strClosestClusterPercID + '</td>\n')
	outfile.write('\t' + strColorLine + '\n')	
	outfile.write('\t</tr>\n')	

	outfile.write('\t<tr>\n')	
	outfile.write('\t<th>Cluster size:</th>\n')	
	outfile.write('\t<td>' + strClusterSize + ' Genomes</td>\n')
	#something to write in the 3rd column?
	outfile.write('\t<td></td>\n')	
	outfile.write('\t</tr>\n')		

#	outfile.write('\t<tr>\n')
#	outfile.write('\t<th>Download extracted marker genes<br> for post analyis:</th>\n')	
#	outfile.write('\t<td><a href="' + strProteinLink + '">Protein Sequences</a></td>\n')
#	outfile.write('\t<td><a href="' + strDNALink + '">Nucelotide Sequences</a></td>\n')	
#	outfile.write('\t</tr>\n')		

	outfile.write('\t</table>\n')
	outfile.write('\t<br>\n')	
	outfile.write('\t' + strBottomLine+ '\n')
	outfile.write('\t<br>\n')
	outfile.write('\t</p>\n')	
	outfile.write('</div>\n')
	
	outfile.close()

def writeMoreResults(dictClosestClusterOGDistances ,dictClosestClusterGenomeDistances, dictGenome2Taxonomy, dictCutoffs, listTypeStrains, outfileName):
	outfile = open(outfileName, "a")

	outfile.write('<div id="moreResults">\n')
	
	#COG results
	outfile.write('\t<div id="cogResults">\n')
	outfile.write('\t<table cellpadding="5">\n')
	
	outfile.write('\t\t<tr>\n')
	outfile.write('\t\t<th>COG</th>\n')
	outfile.write('\t\t<th>Average % Identity</th>\n')	
	outfile.write('\t\t<th>Species cutoff</th>\n')	
	outfile.write('\t\t</tr>\n')
	
	for strCOG in dictClosestClusterOGDistances:
		floatAverageOGClusterDistance = dictClosestClusterOGDistances[strCOG]
		floatAverageOGClusterPercID = convertDistance2PercID(floatAverageOGClusterDistance)	
		strAverageOGClusterPercID = str(floatAverageOGClusterPercID)
		
		floatCutoff = dictCutoffs[strCOG]
		
		strColorLine = '<td style="background-color:Salmon;">Below cutoff</td>'
		if (floatAverageOGClusterDistance <= floatCutoff):
			strColorLine = '<td style="background-color:lightgreen;">Above cutoff</td>'
	
		outfile.write('\t\t<tr>\n')	
		outfile.write('\t\t<td>' + strCOG + '</td>\n')
		outfile.write('\t\t<td>' + strAverageOGClusterPercID + '</td>\n')	
		outfile.write('\t\t' + strColorLine + '\n')	
		outfile.write('\t\t</tr>\n')
		
	outfile.write('\t</table>\n')	
	outfile.write('\t</div>\n')
		
	#taxa results		

	outfile.write('\t<div id="taxaResults">\n')
	outfile.write('\t<table cellpadding="5">\n')
	
	outfile.write('\t\t<tr>\n')
	outfile.write('\t\t<th>NCBI Taxonomy ID</th>\n')
	outfile.write('\t\t<th>NCBI Genome Project ID</th>\n')
	outfile.write('\t\t<th>NCBI Taxonomy Name</th>\n')	
	outfile.write('\t\t<th>Type Strain</th>\n')
	outfile.write('\t\t<th>Average % Identity</th>\n')	
	outfile.write('\t\t<th>Species cutoff</th>\n')	
	outfile.write('\t\t</tr>\n')
	
	for strTaxIDProjectID in dictClosestClusterGenomeDistances:
		floatAverageGenomeClusterDistance = dictClosestClusterGenomeDistances[strTaxIDProjectID]
		floatAverageGenomeClusterPercID = convertDistance2PercID(floatAverageGenomeClusterDistance)	
		strAverageGenomeClusterPercID = str(floatAverageGenomeClusterPercID)
		
		floatCutoff = dictCutoffs[strCOG]
		
		strColorLine = '<td style="background-color:Salmon;">Below cutoff</td>'
		if (floatAverageGenomeClusterDistance <= floatCutoff):
			strColorLine = '<td style="background-color:lightgreen;">Above cutoff</td>'
	
		arrTaxIDProjectID = strTaxIDProjectID.split(".")
	
		strTaxID = arrTaxIDProjectID[0]
		strProjectID = arrTaxIDProjectID[1]
		
		strNCBITaxName = dictGenome2Taxonomy[strTaxID]
		
		strIsTypeStrain = "No"
		if strTaxID in listTypeStrains:
			strIsTypeStrain = "Yes"
		
		outfile.write('\t\t<tr>\n')	
		outfile.write('\t\t<td>' + strTaxID + '</td>\n')
		outfile.write('\t\t<td>' + strProjectID + '</td>\n')
		outfile.write('\t\t<td>' + strNCBITaxName + '</td>\n')
		outfile.write('\t\t<td>' + strIsTypeStrain + '</td>\n')		
		outfile.write('\t\t<td>' + strAverageGenomeClusterPercID + '</td>\n')	
		outfile.write('\t\t' + strColorLine + '\n')	
		outfile.write('\t\t</tr>\n')	
	
	outfile.write('\t</table>\n')	
	outfile.write('\t</div>\n')	
	
	outfile.write('</div>\n')	
	outfile.close()
	
def closeWebsite(outfileName):
	outfile = open(outfileName, "a")	
	outfile.write('</body>\n')
	outfile.write('</html>\n')	
	outfile.close()

	
def writeOverviewResultsNoHit(outfileName):

	outfile = open(outfileName, "a")	

	strColorLine ='<td style="background-color:Salmon;">Below cutoff</td>'
	strBottomLine = '<font size="+1"> Genome could not be assigned to a species cluster.</font>\n'
	strBottomLine2 = '<font size="+1">Additionally, no closely related sequence could be found in the database. Your genome might originate from a novel genus</font>\n'
	#write html code
#	outfile.write('\t<tr>\n')
#	outfile.write('\t<th>Download extracted marker genes<br> for post analyis:</th>\n')	
#	outfile.write('\t<td><a href="' + strProteinLink + '">Protein Sequences</a></td>\n')
#	outfile.write('\t<td><a href="' + strDNALink + '">Nucelotide Sequences</a></td>\n')	
#	outfile.write('\t</tr>\n')		

	outfile.write('<div id="overviewResults">\n')
	outfile.write('\t</table>\n')
	outfile.write('\t<br>\n')	
	outfile.write('\t' + strBottomLine+ '\n')
	outfile.write('\t<br>\n')
	outfile.write('\t<br>\n')	
	outfile.write('\t' + strBottomLine2+ '\n')
	outfile.write('\t<br>\n')
	outfile.write('\t</p>\n')	
	outfile.write('</div>\n')
	
	outfile.close()		
				
	

def printSimpleOverviewResults(outfileName, runName, strClosestCluster, floatClosestClusterDistance, dictCluster2Taxonomy, dictCluster2IDs, dictCutoffs):
	outfile = open(outfileName, "w")	

	strClusterTaxonomy = dictCluster2Taxonomy[strClosestCluster]
	strClosestClusterDistance = str(floatClosestClusterDistance)
	floatCutoff = dictCutoffs["all"]
	strCutoff = str(floatCutoff)
	floatClosestClusterPercID = convertDistance2PercID(floatClosestClusterDistance)	
	strClosestClusterPercID = str(floatClosestClusterPercID)	
	strClusterSize = str(len(dictCluster2IDs[strClosestCluster]))


	strAssigned = "UNASSINGED"
	if (floatClosestClusterDistance <= floatCutoff):
		strAssigned = "FOUND"
	
	strWriteLine = runName + "\t" + strAssigned + "\t" + strClosestCluster + "\t" + strClusterTaxonomy + "\t" + strClosestClusterPercID
	outfile.write(strWriteLine + '\n')	
	outfile.close()

def printSimpleOverviewResultsNoHit(outfileName, runName):
	outfile = open(outfileName, "w")	

	strClosestCluster = "-"
	strClusterTaxonomy = "-"
	strClosestClusterPercID = "-"
	
	strAssigned = "UNASSINGED"

	
	strWriteLine = runName + "\t" + strAssigned + "\t" + strClosestCluster + "\t" + strClusterTaxonomy + "\t" + strClosestClusterPercID
	outfile.write(strWriteLine + '\n')	
	outfile.close()

		
listOfCOGs = ['COG0012','COG0016','COG0018','COG0048','COG0049','COG0052','COG0080','COG0081','COG0085','COG0087','COG0088','COG0090','COG0091','COG0092','COG0093','COG0094','COG0096','COG0097','COG0098','COG0099','COG0100','COG0102','COG0103','COG0124','COG0172','COG0184','COG0185','COG0186','COG0197','COG0200','COG0201','COG0202','COG0215','COG0256','COG0495','COG0522','COG0525','COG0533','COG0541','COG0552']		


		
def runWebserver(infileGenes, infileProteins, geneLengthFileName, clusterFileName, strDB_OGfolder, DBfolder, cutoffFileName, strClusterTaxonomyFileName, strGenomeTaxonomyFileName, strTypeStrainFileName, threads, baseFolder, runName):

	dictCOG2Gene2Length = defaultdict(lambda: defaultdict(int))
#changing input files
	#fileGenes
	#fileProteins

#static input files/ parameters
	#geneLengthFileName
	#strDB_OGfolder	
	#clusterFileName
	#DBfolder
	#cutoffFileName
	#threads
	
#per run folders
	runFolder = baseFolder + "/" + runName + "/"
	os.mkdir(runFolder)

	outfolderTempFasta = baseFolder + "/" + runName + "/" + runName + "_tempFasta"
	os.mkdir(outfolderTempFasta)
	
	outfolderMGsFasta = baseFolder + "/" + runName + "/" + runName + "_MGsFasta"
	os.mkdir(outfolderMGsFasta)
	
	usearchFolder = baseFolder + "/" + runName + "/" + runName + "_usearch"
	os.mkdir(usearchFolder)
	
	distFolder = baseFolder + "/" + runName + "/" + runName + "_dists"
	os.mkdir(distFolder)
	
	print("Parsing utility files")
	dictCutoffs = parseCutoffs(cutoffFileName)
	(dictCOG2Gene2Length) = parseGene2Lengths(geneLengthFileName, dictCOG2Gene2Length)
	(dictID2Cluster, dictCluster2IDs) = fillClusterDict(clusterFileName)
	
	dictCluster2Taxonomy = parseTaxonomyMapping(strClusterTaxonomyFileName)
	dictGenome2Taxonomy = parseTaxonomyMapping(strGenomeTaxonomyFileName)
	dictTypeStrainTaxID2Name = parseTaxonomyMapping(strTypeStrainFileName)
	listTypeStrainTaxIDS = dictTypeStrainTaxID2Name.keys()	
	print("...done")
	
	print("Extracting Marker Genes")
	fileGenes = outfolderTempFasta + "/" + "INPUT.fna"
	fileProteins = outfolderTempFasta + "/" + "INPUT.faa"
	dictNewNames2OldNames = reformatGenome(infileGenes, infileProteins, fileGenes, fileProteins)
	runFetchMGs(fileGenes, fileProteins, outfolderMGsFasta, threads)
	(dictCOG2Gene2Length) = getGeneLengthInput(outfolderMGsFasta, listOfCOGs, dictCOG2Gene2Length)	
	print("...done")
	
	print("Determining distances")	
	listHitTaxIDProjects = runUsearchPreScreen(listOfCOGs, outfolderMGsFasta, usearchFolder, strDB_OGfolder, threads)
	if (listHitTaxIDProjects):
		(listHitTaxIDProjects_FullClusters, listHitClusters) = getAllClusterMembers(listHitTaxIDProjects, dictID2Cluster, dictCluster2IDs)
		(dictInput_ID2COG2Distance, dictInput_ID2COG2DistanceWeight) = runGLsearch(listHitTaxIDProjects_FullClusters, outfolderMGsFasta, listOfCOGs, DBfolder, distFolder, dictCOG2Gene2Length, threads)
		print("...done")
		
		print("Trying to assign to cluster")
		(strClosestCluster, floatClosestClusterDistance, dictClosestClusterGenomeDistances, dictClosestClusterOGDistances) = calcDist2Clusters( dictInput_ID2COG2Distance, dictInput_ID2COG2DistanceWeight, listHitTaxIDProjects_FullClusters, listHitClusters, dictCluster2IDs )
		print("...done")

		print("Making website")	
		outfileName = runFolder + "/" + runName + ".html"
		
		writeHeader(outfileName)
		writeOverviewResults(outfileName, strClosestCluster, floatClosestClusterDistance, dictCluster2Taxonomy, dictCluster2IDs, dictCutoffs)
		writeMoreResults(dictClosestClusterOGDistances ,dictClosestClusterGenomeDistances, dictGenome2Taxonomy, dictCutoffs, listTypeStrainTaxIDS, outfileName)
		closeWebsite(outfileName)
		print("...done")
		shutil.copy(binFolder + "/data/main.css", runFolder)

		outfileName = runFolder + "/" + runName + ".results"
		printSimpleOverviewResults(outfileName, runName, strClosestCluster, floatClosestClusterDistance, dictCluster2Taxonomy, dictCluster2IDs, dictCutoffs)
	else:
		print("...done")	
		print("Making website")
		outfileName = runFolder + "/" + runName + ".html"
		writeHeader(outfileName)
		writeOverviewResultsNoHit(outfileName)
		closeWebsite(outfileName)		
		print("...done")
	
		shutil.copy(binFolder + "/data/main.css", runFolder)

		outfileName = runFolder + "/" + runName + ".results"
		printSimpleOverviewResultsNoHit(outfileName, runName)
	
	
###change this from here ...
def printUsage():
	print("Usage: specI.py infileGenes infileProteins threads runName basefolder")
	print("infileGenes: fasta file")
	print("infileProteins: fasta file")
	print("threads: number of threads to be used")
	print("runName: name of current run")
	print("basefolder: basefolder of current run")
	
def main(argv=None):
	argv = sys.argv[1:]
	
	baseFolder = "."
	threads = "1"
	
	if (len(argv) == 5):
		infileGenes = argv[0]
		infileProteins = argv[1]
		threads = argv[2]
		runName = argv[3]
		baseFolder = argv[4]
	else:
		printUsage()
		sys.exit()
	
	#/g/bork2/mende/taxonomy_cutoffs/revision/comp_evalulation/amphora/clustering
	
	
	geneLengthFileName = binFolder + "/data/all.project.lengths"
	clusterFileName = binFolder + "/data/weightedMean.dna.average.0.035.clustering"
	strDB_OGfolder = binFolder + "/sequences/"
	DBfolder = strDB_OGfolder
	cutoffFileName = binFolder + "/data/cutoffs_all.txt" 
	
	strClusterTaxonomyFileName = binFolder + "/data/weightedMean.dna.average.0.035.clustering.annotated.curated"
	strGenomeTaxonomyFileName = binFolder + "/data/RefOrganismDB.v9.taxids2names"
	strTypeStrainFileName = binFolder + "/data/typeStrains_freeze9.txt"

	runWebserver(infileGenes, infileProteins, geneLengthFileName, clusterFileName, strDB_OGfolder, DBfolder, cutoffFileName, strClusterTaxonomyFileName, strGenomeTaxonomyFileName, strTypeStrainFileName, threads, baseFolder, runName)
	return 0        # success

	
if __name__ == '__main__':
    status = main()
    sys.exit(status)		